#!/usr/bin/python3

"""This module takes care of all logging related needs"""

# Thanks to
# https://uran198.github.io/en/python/2016/07/12/colorful-python-logging.html

import logging
import colorama
import copy

# specify colors for different logging levels
LOG_COLORS = {
    logging.ERROR: colorama.Fore.RED,
    logging.CRITICAL: colorama.Fore.RED,
    logging.WARNING: colorama.Fore.YELLOW
}


class ColorFormatter(logging.Formatter):
    def format(self, record, *args, **kwargs):
        # if the corresponding logger has children, they may receive modified
        # record, so we want to keep it intact
        new_record = copy.copy(record)
        if new_record.levelno in LOG_COLORS:
            # we want levelname to be in different color, so let's modify it
            new_record.msg = "{color_begin}{msg}{color_end}".format(
                msg=new_record.msg,
                color_begin=LOG_COLORS[new_record.levelno],
                color_end=colorama.Style.RESET_ALL,
            )
            new_record.levelname = "{color_begin}{level}{color_end}".format(
                level=new_record.levelname,
                color_begin=LOG_COLORS[new_record.levelno],
                color_end=colorama.Style.RESET_ALL,
            )
        # now we can let standart formatting take care of the rest
        return super().format(new_record, *args, **kwargs)



#!/usr/bin/python3

"""This module contains all the selectors used"""

from enum import Enum


class Jvc(Enum):
    """The jeuxvideo.com selectors"""
    SITE = "jeuxvideo.com"
    AUTHOR_SELECTOR = ".bloc-header > span"
    DELETED_AUTHOR_SELECTOR = ".bloc-pseudo-msg"
    TOTAL_SELECTOR = ".bloc-liste-num-page > span:nth-last-of-type(1)"
    TOTAL_SELECTOR_ALTERNATIVE = (
        ".bloc-liste-num-page > span:nth-last-of-type(2)"
    )
    MESSAGE_SELECTOR = ".conteneur-message"
    RISITAS_TEXT_SELECTOR = "[class='txt-msg text-enrichi-forum']"
    TITLE_SELECTOR = "#bloc-title-forum"
    NOELSHACK_IMG_SELECTOR = "img.img-shack"
    PAGE_TITLE_SELECTOR = "title"


class Jvarchive(Enum):
    """The jvarchive.com selectors"""
    SITE = "jvarchive.com"
    AUTHOR_SELECTOR = (
        "[class='h4 text-truncate min-width-0 mb-0 text-primary'] > a"
    )
    DELETED_AUTHOR_SELECTOR = "[class='h4 text-truncate min-width-0 mb-0']"
    TOTAL_SELECTOR = ".page-item:nth-last-child(2)"
    MESSAGE_SELECTOR = "[class='card-body pb-0 px-3 px-md-4 pt-3']"
    RISITAS_TEXT_SELECTOR = ".conteneur-message"
    TITLE_SELECTOR = "[class='h2 text-white d-inline align-middle mb-0 mr-2']"
    NOELSHACK_IMG_SELECTOR = "img"
    PAGE_TITLE_SELECTOR = "title"


class Webarchive(Enum):
    """The web.archive.org selectors"""
    SITE = "web.archive.org"
    AUTHOR_SELECTOR = "h2 + p > a"
    TOTAL_SELECTOR = ".page-links > a:last-child"
    MESSAGE_SELECTOR = ".entry-content"
    RISITAS_TEXT_SELECTOR = ".entry-content > div.txt-msg.text-enrichi-forum"
    RISITAS_TEXT_SELECTOR_ALTERNATIVE = ".entry-content > div"
    RISITAS_TEXT_SELECTOR_ALTERNATIVE2 = (
        ".entry-content >  h2:last-of-type ~ p"
    )
    RISITAS_TEXT_SELECTOR_ALTERNATIVE3 = (
        ".entry-content >  p"
    )  # Need to select all
    TITLE_SELECTOR = ".entry-content > h2"
    NOELSHACK_IMG_SELECTOR = ".entry-content > div > p > a > img"
    PAGE_TITLE_SELECTOR = "title"


class Noelshack(Enum):
    """The noelshack.com selectors"""
    IMG_SELECTOR = "#elt_to_aff"



#!/usr/bin/python3

"""This is the main module containing the core routines for risiparse"""

from typing import List, Optional
import sys
import logging
import datetime
import pathlib
import re
import requests
import waybackpy

from bs4 import BeautifulSoup
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry
from risiparse.sites_selectors import Noelshack, Jvc, Jvarchive, Webarchive
from risiparse.utils.utils import (
    write_html,
    append_html,
    get_domain,
    make_dirs,
    get_selectors_and_site,
    create_pdfs,
    parse_input_links,
    get_args,
    strip_webarchive_link,
    replace_youtube_frames,
    contains_webarchive
)
from risiparse.utils.log import ColorFormatter
from risiparse.utils.database import update_db, read_db, delete_db

LOGGER = logging.getLogger()
LOGGER.setLevel(logging.INFO)

FMT = '%(asctime)s:%(levelname)s: %(message)s'

STDOUT_HANDLER = logging.StreamHandler()
STDOUT_HANDLER.setLevel(logging.INFO)
STDOUT_HANDLER.setFormatter(ColorFormatter(FMT))

TODAY = datetime.date.today()

LOGGER.addHandler(STDOUT_HANDLER)

DEFAULT_TIMEOUT = 5  # seconds


class TimeoutHTTPAdapter(HTTPAdapter):
    """This takes care of the default timeout for all requests"""
    def __init__(self, *args, **kwargs):
        self.timeout = DEFAULT_TIMEOUT
        if "timeout" in kwargs:
            self.timeout = kwargs["timeout"]
            del kwargs["timeout"]
        super().__init__(*args, **kwargs)

    def send(self, request, **kwargs):
        timeout = kwargs.get("timeout")
        if timeout is None:
            kwargs["timeout"] = self.timeout
        return super().send(request, **kwargs)


class PageDownloader():
    """Handle all the downloads made by risiparse"""

    def __init__(self, domain: str):
        self.domain = domain
        self.webarchive = bool(self.domain == Webarchive.SITE.value)
        self.http = requests.Session()
        self.retries = Retry(
            total=5,
            status_forcelist=[429, 500, 502, 503, 504],
            backoff_factor=1
        )
        self.adapter = TimeoutHTTPAdapter(max_retries=self.retries)
        self.http.mount("https://", self.adapter)

    def download_topic_page(
            self,
            page_link: str,
            page_number: int = 1,
    ) -> Optional['BeautifulSoup']:
        if not self.webarchive:
            page_link = self._get_page_link(
                page_number,
                page_link
            )
        else:
            webarchive_link = self._get_webarchive_page_link(
                page_number,
                page_link
            )
            if webarchive_link:
                page_link = webarchive_link
        logging.info(f"Going to page {page_link}")
        try:
            page = self.http.get(page_link)
        except requests.exceptions.RetryError as e:
            logging.exception(e)
            logging.error(
                f"The retries for {page_link} have failed, "
                f"being rate limited/server overloaded"
            )
            return None
        page_status = page.status_code
        if page_status == 410:
            logging.error(
                "The page has been 410ed, try "
                "with one from jvarchive or webarchive"
            )
        if self.webarchive and page_status == 404:
            logging.error(
                f"The Wayback Machine has not archived "
                f"{page_link}"
            )
            return None
        soup = BeautifulSoup(page.content, features="lxml")
        return soup

    def download_img_page(self, page_link: str) -> str:
        page = self.http.get(page_link)
        soup = BeautifulSoup(page.content, features="lxml")
        img_link = soup.select_one(
            Noelshack.IMG_SELECTOR.value
        ).attrs["src"]
        return img_link

    def _change_src_path(
            self,
            img: 'BeautifulSoup',
            img_folder_path: pathlib.Path,
            file_name: str
    ) -> None:
        img.attrs["src"] = str(img_folder_path) + "/" + file_name

    def _image_exists(
            self,
            file_name: str,
            img_folder: pathlib.Path
    ) -> bool:
        img_path = img_folder / file_name
        return bool(img_path.exists())

    def _get_page_link(
            self,
            page_number: int,
            page_link: str,
    ) -> str:
        page_link_number = [
               m.span() for m in re.finditer(r"\d*\d", page_link)
        ]
        page_link = (
               page_link[:page_link_number[3][0]]
               + str(page_number) +
               page_link[page_link_number[3][1]:]
        )
        return page_link

    def _get_webarchive_page_link(
            self,
            page_number: int,
            page_link: str
    ) -> Optional[str]:
        if page_number == 1:
            return None
        page_link_splitted = page_link.rsplit("/")
        if len(page_link) == 11:
            page_link = (
                "/".join(page_link_splitted[:-2]) + f"/{page_number}/"
            )
        else:
            page_link = (
                "/".join(page_link_splitted[:-1]) + f"/{page_number}/"
            )
        return page_link

    def get_webarchive_img(
            self,
            link: str,
    ) -> Optional['requests.models.Response']:
        logging.error(
            f"The image at {link} "
            "has been 404ed! Trying on "
            "webarchive..."
        )
        user_agent = (
            "Mozilla/5.0 (Windows NT 5.1; rv:40.0) "
            "Gecko/20100101 Firefox/40.0"
        )
        try:
            wayback = waybackpy.Url(link, user_agent)
            oldest_archive = wayback.oldest()
            oldest_archive_url = oldest_archive.archive_url
            image = self.http.get(oldest_archive_url)
            status_code = image.status_code
        except waybackpy.exceptions.WaybackError as e:
            logging.exception(e)
            return None
        if status_code != 200:
            logging.error(
                f"The image at {oldest_archive_url} "
                "on the oldest archive could "
                "not been downloaded"
            )
            return None
        return image

    def get_webarchive_link(
            self,
            img: BeautifulSoup
    ) -> str:
        archive_link = img.attrs["src"]
        contains_webarchive_link = contains_webarchive(
            archive_link
        )
        if contains_webarchive_link:
            link = strip_webarchive_link(archive_link)
        else:
            link = archive_link
        return link

    def download_images(
            self,
            soup: BeautifulSoup,
            output_dir: pathlib.Path,
    ) -> None:
        img_folder_path = output_dir / "risitas-html" / "images"
        img_folder_path.mkdir(exist_ok=True)
        for page in soup:
            imgs = page[0].select("img")
            for img in imgs:
                if self.webarchive:
                    link = self.get_webarchive_link(img)
                else:
                    link = img.attrs["src"]
                file_name = link[link.rfind("/"):][1:]
                if not self._image_exists(file_name, img_folder_path):
                    logging.info(
                        "Image not in cache, downloading "
                        f"{file_name}"
                    )
                    try:
                        image = self.http.get(link)
                        status_code = image.status_code
                    # Connection refused or others errors.
                    except requests.exceptions.ConnectionError as e:
                        logging.exception(e)
                        continue
                    if status_code == 404:
                        image = self.get_webarchive_img(link)
                        if not image:
                            continue
                    file_name = image.url[image.url.rfind("/"):][1:]
                    img_file_path = img_folder_path / file_name
                    img_file_path.write_bytes(image.content)
                self._change_src_path(img, img_folder_path, file_name)


class RisitasInfo():
    """
    This gets the author name and the total number of pages and the title.
    """

    def __init__(self, page_soup: BeautifulSoup, selectors, domain: str):
        self.soup = page_soup
        self.selectors = selectors
        self.domain = domain
        self.author = self.get_author_name(self.soup)
        self.total_pages = self.get_total_pages(self.soup)
        self.title = self.get_title(self.soup)

    def get_author_name(self, soup: BeautifulSoup) -> str:
        if self.domain == "jeuxvideo.com":
            author = soup.select_one(
                self.selectors.DELETED_AUTHOR_SELECTOR.value
            ).text.strip()
            if author == "Pseudo supprimé":
                logging.error(
                    "The author has deleted his account, "
                    "need to sort the message after this "
                    "and look if he posted with an "
                    "other account"
                )
                return author
        try:
            author = soup.select_one(
                self.selectors.AUTHOR_SELECTOR.value
            ).text.strip()
        except AttributeError as e:
            author = "unknown"
            logging.exception(e)
            logging.error(
                "Can't get the risitas "
                f"author, set author to '{author}'"
            )
        if not author and self.domain == Webarchive.SITE.value:
            author = "Pseudo supprimé"
        logging.info(
            f"The risitas author is : {author}"
        )
        return author

    def get_total_pages(self, soup: BeautifulSoup) -> int:
        try:
            topic_symbol = soup.select_one(
                self.selectors.TOTAL_SELECTOR.value
            ).text
            if self.domain == Jvc.SITE.value:
                if topic_symbol == "»":
                    topic_symbol = soup.select_one(
                        self.selectors.TOTAL_SELECTOR_ALTERNATIVE.value
                    ).text
        except AttributeError:
            topic_symbol = None
        if not topic_symbol and self.domain == Webarchive.SITE.value:
            logging.info(
                "This risitas has only one "
                "page!"
            )
            topic_pages = 1
        else:
            topic_pages = int(topic_symbol)
        return topic_pages

    def get_title(self, soup: BeautifulSoup) -> str:
        try:
            title = soup.select_one(
                self.selectors.TITLE_SELECTOR.value
            ).text.strip()
        except AttributeError as e:
            logging.exception(e)
            logging.error(
                "Can't get the title "
                "author, setting the title to "
                "the page title"
            )
            title = soup.select_one(
                self.selectors.PAGE_TITLE_SELECTOR.value
            ).text.strip()
        return title


class Posts():
    """Get a post author and content"""

    def __init__(
            self,
            selectors: BeautifulSoup,
            downloader: PageDownloader,
            domain: str,
            all_messages: bool = False,
            no_resize_images: bool = False,
    ):
        self.risitas_html: List['BeautifulSoup'] = []
        self.risitas_raw_text: List[str] = []
        self.count = 0
        self.duplicates = 0
        self.selectors = selectors
        self.all_messages = all_messages
        self.downloader = downloader
        self.no_resize_images = no_resize_images
        self.authors: List[str] = []
        self.domain = domain
        self.webarchive = bool(domain == Webarchive.SITE.value)
        self.past_message = False
        self.message_cursor = 0

    def _check_post_author(
            self,
            post: BeautifulSoup,
            no_match_author: bool,
    ) -> bool:
        # This is needed cuz deleted accounts are not handled
        # the same way for whatever reason...
        try:
            post_author = post.select_one(
                self.selectors.AUTHOR_SELECTOR.value
            ).text.strip()
        except AttributeError:
            if not self.webarchive:
                logging.debug("Author deleted his account")
                post_author = post.select_one(
                    self.selectors.DELETED_AUTHOR_SELECTOR.value
                ).text.strip()
            else:
                return False
        logging.debug(
            "The current author is "
            "{post_author} "
            "and the risitas author is "
            "{self.authors}"
        )
        if not no_match_author:
            for author in self.authors:
                author_root = re.sub(r"\d*", "", author)
                if author_root in post_author:
                    return True
        return bool(post_author in self.authors)

    def _check_post_length(self, post: BeautifulSoup) -> bool:
        """Check the post length to see if this is a chapter
        or an offtopic message"""
        p = post.text.strip().replace("\n", "")
        return bool(len(p) < 1000)

    def _check_post_identifiers(
            self,
            post: BeautifulSoup,
            identifiers: List
    ) -> bool:
        identifiers_joined = "|".join(identifiers)
        regexp = re.compile(identifiers_joined, re.IGNORECASE)
        try:
            post_paragraph_text = post.select_one("p").text[0:200]
        except AttributeError as e:
            logging.exception(
                f"The post doesn't contains text, probably some image {e}"
            )
            return False
        contains_identifiers = regexp.search(post_paragraph_text)
        return bool(
            contains_identifiers and not post.select_one("blockquote")
        )

    def _check_post_duplicates(
            self,
            risitas_html: 'BeautifulSoup',
            contains_images: bool
    ) -> bool:
        if not self.risitas_raw_text:
            return False
        ret_val = False
        if contains_images:
            for part in self.risitas_html:
                if part[1] is True and risitas_html == part[0]:
                    ret_val = True
                    break
            return ret_val
        for i in range(len(self.risitas_raw_text)):
            ret_val = bool(
                risitas_html.text.lower() ==
                self.risitas_raw_text[-i].lower() or
                risitas_html.text[0:100].lower() ==
                self.risitas_raw_text[-i][0:100].lower()
            )
        return ret_val

    def _contains_blockquote(self, risitas_html: BeautifulSoup) -> bool:
        return bool(risitas_html.select("blockquote"))

    def _check_chapter_image(self, soup: BeautifulSoup) -> bool:
        ret_val = False
        paragraphs = soup.select("p")
        if not paragraphs and self.domain == Jvarchive.SITE.value:
            a = soup.select("a")
            if a:
                return True
        # Need to check alt startswith https
        for paragraph in paragraphs:
            if re.search(r"screen|repost|supprime", paragraph.text):
                if (
                        len(
                            soup.select(
                                self.selectors.NOELSHACK_IMG_SELECTOR.value
                            )
                        ) >= 3 and not soup.select("blockquote") and
                        len(soup.text.strip()) < 300

                ):
                    ret_val = True
                    break
            elif not paragraph.text.strip():
                if (
                        len(
                            soup.select(
                                self.selectors.NOELSHACK_IMG_SELECTOR.value
                            )
                        ) >= 3 and not soup.select("blockquote") and
                        len(soup.text.strip()) < 300
                ):
                    ret_val = True
            else:
                ret_val = False
        return ret_val

    def _get_fullscale_image(self, soup: BeautifulSoup) -> BeautifulSoup:
        image_soup = soup
        imgs = image_soup.select(self.selectors.NOELSHACK_IMG_SELECTOR.value)
        remove_attrs = ["width", "height"]
        for remove_attr in remove_attrs:
            for img in imgs:
                try:
                    img.attrs.pop(remove_attr)
                    logging.info(
                        f"Displaying {img.attrs['src']} at full scale!"
                    )
                except KeyError as e:
                    logging.exception(f"This is a jvc smiley! {e}")
        if self.domain == Jvc.SITE.value:
            for img in imgs:
                if re.search("fichiers", img.attrs["alt"]):
                    img.attrs["src"] = img.attrs["alt"]
                else:
                    img.attrs["src"] = self.downloader.download_img_page(
                        img.attrs["alt"]
                    )
        elif self.domain == Jvarchive.SITE.value:
            for img in imgs:
                img.attrs["src"] = img.attrs["alt"]
        return image_soup

    def _print_chapter_added(
            self,
            risitas_html: BeautifulSoup
    ) -> None:
        try:
            pretty_print = risitas_html.select_one(
                'p'
            ).text[0:50].strip().replace("\n", "")
            if not pretty_print and len(risitas_html.select("p")) > 1:
                pretty_print = risitas_html.select("p")[1].text[
                    0:50].strip().replace("\n", "")
            if not pretty_print:
                logging.debug(
                    "Added some images, maybe chapters in screenshot?"
                )
            else:
                logging.info(
                    "Adding "
                    f"{pretty_print}"
                )
        except AttributeError as e:
            logging.exception(
                f"Can't log text because this "
                f"is an image from jvarchive {e}"
            )

    def _get_webarchive_post_html(
            self,
            post: BeautifulSoup
    ) -> BeautifulSoup:
        new_html = []
        risitas_html = post.select_one(
            self.selectors.RISITAS_TEXT_SELECTOR_ALTERNATIVE.value
        )
        if risitas_html and risitas_html.p:
            return risitas_html
        risitas_html = post.select(
               self.selectors.RISITAS_TEXT_SELECTOR_ALTERNATIVE2.value
        )
        if not risitas_html:
            risitas_html = post.select(
                self.selectors.RISITAS_TEXT_SELECTOR_ALTERNATIVE3.value
            )
        for paragraph in risitas_html:
            new_html.append(str(paragraph))
            new_html_joined = "".join(new_html)
            risitas_html = BeautifulSoup(new_html_joined, features="lxml")
            # This is needed cuz beautifulsoup
            # add a html,body tag automatically
        try:
            risitas_html.html.wrap(
                risitas_html.new_tag(
                    "div",
                    attrs={"class": "txt-msg text-enrichi-forum"}
                )
            )
            risitas_html.html.body.unwrap()
            risitas_html.html.unwrap()
        except AttributeError:
            pass
        return risitas_html

    def get_posts(
            self,
            soup: BeautifulSoup,
            risitas_authors: List,
            identifiers: List,
            no_match_author: bool,
            append: bool,
            message_cursor_db: int,
    ) -> None:
        posts = soup.select(self.selectors.MESSAGE_SELECTOR.value)
        # Counter necessary to keep the first post in risitas
        # This post usually have no identifiers and is used
        # as an intro
        if not self.authors:
            self.authors = risitas_authors
        self.added_message = False
        for message_cursor, post in enumerate(posts):
            # We go to the last page recorded in the db
            # And we skip the messages according to the cursor
            # position in the database
            if append and not self.past_message:
                if message_cursor <= message_cursor_db:
                    continue
                else:
                    self.past_message = True
            is_author = self._check_post_author(post, no_match_author)
            risitas_html = post.select_one(
                self.selectors.RISITAS_TEXT_SELECTOR.value
            )
            try:
                contains_paragraph = risitas_html.select("p")
            except AttributeError:
                contains_paragraph = None
            if self.webarchive and not contains_paragraph:
                risitas_html = self._get_webarchive_post_html(post)
            if not is_author and not self.webarchive:
                continue
            contains_image = self._check_chapter_image(risitas_html)
            if contains_image:
                logging.debug("The current post contains images!")
                if not self.no_resize_images:
                    risitas_html = self._get_fullscale_image(risitas_html)
            if self.all_messages:
                self.count += 1
                self.risitas_html.append((risitas_html, ))
                continue
            contains_identifiers = self._check_post_identifiers(
                post,
                identifiers
            )
            is_short = self._check_post_length(post)
            contains_blockquote = self._contains_blockquote(risitas_html)
            if contains_blockquote:
                continue
            if (
                    not contains_identifiers and
                    self.count > 1 and
                    not contains_image
                    and is_short
            ):
                continue
            if is_short and not contains_image:
                continue
            is_duplicate = self._check_post_duplicates(
                risitas_html,
                contains_image
            )
            if is_duplicate:
                logging.info(
                    (
                        "The current post "
                        f"{risitas_html.select_one('p').text[0:50].strip()}"
                        "is a duplicate!"
                    )
                )
                self.duplicates += 1
                continue
            self._print_chapter_added(risitas_html)
            self.added_message = True
            self.risitas_html.append((risitas_html, contains_image))
            self.risitas_raw_text.append(risitas_html.text)
            self.count += 1
            self.message_cursor = message_cursor


def main() -> None:
    args = get_args()
    output_dir = args.output_dir.expanduser().resolve()

    # File logging
    log_file = output_dir / f"risiparse-{TODAY}.log"
    FILE_HANDLER = logging.FileHandler(log_file)
    FILE_HANDLER.setLevel(logging.DEBUG)
    FILE_HANDLER.setFormatter(logging.Formatter(FMT))
    LOGGER.addHandler(FILE_HANDLER)

    all_messages = args.all_messages
    download_images = args.download_images
    identifiers = args.identifiers
    no_match_author = args.no_match_author
    no_resize_images = args.no_resize_images
    clear_database = args.clear_database
    no_database = args.no_database
    create_pdfs_user = args.create_pdfs
    htmls_file_path = None
    if clear_database:
        delete_db()
        sys.exit()
    make_dirs(output_dir)
    if args.debug:
        STDOUT_HANDLER.setLevel(logging.DEBUG)
        FILE_HANDLER.setLevel(logging.DEBUG)
    if not args.no_download:
        page_links = parse_input_links(args.links)
        htmls_file_path = []
        for link in page_links:
            # This part is just used to get the risitas info
            domain = get_domain(link)
            if domain == Webarchive.SITE.value:
                no_database = True
            selectors = get_selectors_and_site(link)
            page_number: int = 1
            page_downloader = PageDownloader(domain)
            soup = page_downloader.download_topic_page(
                link,
                page_number,
            )
            risitas_info = RisitasInfo(soup, selectors, domain)
            authors = [risitas_info.author] + args.authors
            posts = Posts(
                selectors,
                page_downloader,
                domain,
                all_messages,
                no_resize_images,
            )
            total_pages = risitas_info.total_pages
            if not no_database:
                row = read_db(link)
                if row[0]:
                    page_number = 0
                    # +1 to evaluate the page itself
                    total_pages = risitas_info.total_pages - row[0][6] + 1
            for page in range(total_pages):
                append = False
                message_cursor_db = 0
                if not no_database:
                    if row[0]:
                        if not page_number:
                            page_number = row[0][6]
                        append = True
                        message_cursor_db = row[0][7]
                    else:
                        message_cursor_db = 0
                soup = page_downloader.download_topic_page(
                    link, page_number
                )
                if not soup:
                    page_number += 1
                    continue
                posts.get_posts(
                    soup,
                    authors,
                    identifiers,
                    no_match_author,
                    append,
                    message_cursor_db
                )
                page_number += 1
                if (
                        posts.message_cursor and
                        page + 1 == total_pages and
                        posts.added_message
                ):
                    message_cursor = posts.message_cursor
                else:
                    message_cursor = 0
            if not posts.risitas_html and not no_database:
                logging.info("There is no new chapters available!")
                continue
            logging.info(
                "The number of posts downloaded for "
                f"{risitas_info.title} "
                f"is : {posts.count}"
            )
            risitas_html = posts.risitas_html
            if not all_messages:
                logging.info(
                    "The number of duplicates for "
                    f"{risitas_info.title} "
                    f"is : {posts.duplicates}"
                )
            if download_images:
                page_downloader.download_images(
                    risitas_html,
                    output_dir,
                )
            if domain == Webarchive.SITE.value:
                risitas_html = replace_youtube_frames(risitas_html)
            if append and not no_database:
                file_path = pathlib.Path(row[0][4])
                htmls_file_path.append(file_path)
                append_html(file_path, risitas_html)
            else:
                file_path = write_html(
                    risitas_info.title,
                    risitas_info.author,
                    risitas_html,
                    output_dir,
                )
                htmls_file_path.append(file_path)
            if not no_database:
                update_db(
                    domain,
                    risitas_info.title,
                    link,
                    file_path,
                    risitas_info.total_pages,
                    risitas_info.total_pages,
                    message_cursor,
                    authors,
                )
    if not args.no_pdf:
        create_pdfs(output_dir, create_pdfs_user, htmls_file_path)



#!/usr/bin/python3

"""This module just regroup some routines"""

from typing import List, Union
import unicodedata
import pathlib
import re
import sys
import argparse
import logging

from urllib.parse import urlparse
from bs4 import BeautifulSoup
from risiparse import html_to_pdf, sites_selectors


def write_html(
        title: str,
        author: str,
        posts: List,
        output_dir: 'pathlib.Path',
) -> 'pathlib.Path':
    title_slug = slugify(title, title=True)
    author_slug = slugify(author, title=False)
    ext = "html"
    i = 0
    full_title = f"{author_slug}-{title_slug}-{i}"
    html_path = (
        output_dir /
        "risitas-html" /
        (full_title + f".{ext}")
    )
    while html_path.exists():
        i += 1
        full_title = f"{author_slug}-{title_slug}-{i}"
        html_path = (
            output_dir /
            "risitas-html" /
            (full_title + f".{ext}")
        )
    with open(html_path, "w", encoding="utf-8") as f:
        html = (
            """<!DOCTYPE html>
            <html lang='fr'>
            <head>
            <meta charset='UTF-8'>
            <meta name='viewport' \
            content='width=device-width, initial-scale=1.0'>
            <title>Risitas</title>
            </head>
            <body>"""
        )
        f.write(html)
        for paragraph in posts:
            f.write(str(paragraph[0]).replace("’", "'"))
        html = (
            """</body>
            </html>"""
        )
        f.write(html)
        logging.info(f"Wrote {html_path}")
    return html_path


def append_html(html_path: 'pathlib.Path', risitas_html: 'BeautifulSoup'):
    with open(html_path, encoding='utf-8') as f:
        soup = BeautifulSoup(f, features="lxml")
    for new_chapter in risitas_html:
        soup.body.insert(len(soup.body.contents), new_chapter[0])
        logging.debug(f"Appending {new_chapter[0]} ...")
    with open(html_path, "w", encoding='utf-8') as f:
        f.write(soup.decode().replace("’", "'"))
    logging.info(f"The chapters have been appended to {html_path}")


def _remove_risitas(title: str) -> str:
    regexp = re.compile(r"\[risitas\]", re.IGNORECASE)
    return regexp.sub("", title).strip()


def _replace_whitespaces(title: str) -> str:
    title_dashes = title.replace(" ", "-")
    return title_dashes


def slugify(
        string: str,
        allow_unicode: bool = False,
        title: bool = False
) -> str:
    if title:
        string = _remove_risitas(string)
        string = _replace_whitespaces(string)
    if allow_unicode:
        string = unicodedata.normalize('NFKC', string)
    else:
        string = unicodedata.normalize(
            'NFKD', string
        ).encode(
            'ascii', 'ignore'
        ).decode('ascii')
    string = re.sub(r'[^\w\s-]', '', string.lower())
    return re.sub(r'[-\s]+', '-', string).strip('-_')


def _strip_www_component(url: str) -> str:
    domain = re.sub(r"www.", "", url)
    return domain


def get_domain(url: str) -> str:
    domain = url.split("/")[2]
    domain = _strip_www_component(domain)
    return domain


def make_dirs(output_dir: 'pathlib.Path') -> None:
    (output_dir / "risitas-html").mkdir(exist_ok=True, parents=True)
    (output_dir / "risitas-pdf").mkdir(exist_ok=True, parents=True)


def get_selectors_and_site(
        link: str
) -> tuple[
    Union[
        'sites_selectors.Jvc',
        'sites_selectors.Jvarchive',
        'sites_selectors.Webarchive'
    ],
    str
]:
    domain = get_domain(link)
    if domain == "jeuxvideo.com":
        selectors = sites_selectors.Jvc
    elif domain == "jvarchive.com":
        selectors = sites_selectors.Jvarchive
    elif domain == "web.archive.org":
        selectors = sites_selectors.Webarchive
    return selectors


def create_pdfs(
        output_dir: 'pathlib.Path',
        html_user: List['pathlib.Path'] = None,
        html_download: List['pathlib.Path'] = None,
) -> None:
    html_folder_path = output_dir / "risitas-html"
    if not html_user and not html_download:
        htmls = list(html_folder_path.glob("*.html"))
    elif html_user:
        htmls = html_user
    elif html_download:
        htmls = html_download
    if htmls:
        app = html_to_pdf.QtWidgets.QApplication([])
        page = html_to_pdf.PdfPage(output_dir)
        logging.info("Creating pdfs...")
        page.convert(htmls)
        sys.exit(app.exec_())


def read_links(links_file: pathlib.Path) -> List:
    links_file = links_file.expanduser().resolve()
    with open(links_file, encoding='utf-8') as f:
        page_links = [line.strip() for line in f if line.strip() != '']
    return page_links


def get_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    default_links = [pathlib.Path().cwd() / "risitas-links"]
    parser.add_argument(
        "--all-messages",
        action="store_true",
        default=False,
        help=(
            "Download all the messages from the author."
            "Default : False"
        )
    )
    parser.add_argument(
        "--no-pdf",
        action="store_true",
        default=False,
        help="Default : False, only download html"
    )
    parser.add_argument(
        "--debug",
        action="store_true",
        default=False,
        help="Verbose output, Default : False"
    )
    parser.add_argument(
        "--no-download",
        action="store_true",
        default=False,
        help=(
            "Default : False, Create pdfs directly from current dir "
            "or one specified by -o"
        )
    )
    # Links
    parser.add_argument(
        "-l", "--links",
        action="store",
        nargs='+',
        default=default_links,
        help=(
            "The links file, or links from standard input, "
            "Default : current dir/risitas-links"
        )
    )
    # Pdfs to create
    parser.add_argument(
        "--create-pdfs",
        nargs='+',
        help=(
            "A list of html files path to create pdfs from "
            "If this option is not specified with --no-download "
            "the pdfs will be created for all html files in risitas-html"
        ),
        type=lambda p: pathlib.Path(p).expanduser().resolve()
    )
    # Take a list of identifiers
    parser.add_argument(
        '-i',
        '--identifiers',
        nargs='+',
        help=(
            "Give a list of words that are going to be matched by the script, "
            "example: a message that has the keyword 'hors-sujet', "
            "by adding 'hors-sujet' with this option, "
            "the script will match the message that has this keyword. "
            "Default : 'chapitre'"
        ),
        required=False,
        default=["chapitre"],
        type=str
    )
    # Authors if author has multiple accounts
    parser.add_argument(
        '--authors',
        nargs='+',
        help=(
            "List of authors to be matched, by default the author of "
            "the first post author is considered as the author "
            "throughout the whole risitas, "
            "Default : Empty"
        ),
        required=False,
        default=[],
        type=str
    )
    # Images
    parser.add_argument(
        '--no-resize-images',
        help=(
            "When the script 'thinks' that the post contains images "
            "and that they are chapters posted in screenshot, "
            "it will try to display them to their full width, "
            "Default : False"
        ),
        action="store_true",
        required=False,
        default=False
    )
    parser.add_argument(
        '--download-images',
        help=(
            "Whether to download images locally "
            "If set, this will change all img[src] link to point "
            "to the local images "
            "Also this will try to download risitas sticker on webarchive "
            "if they have been 404ed, "
            "Default : False"
        ),
        action="store_true",
        required=False,
        default=False
    )
    # Match author
    parser.add_argument(
        "--no-match-author",
        action="store_true",
        default=False,
        help=(
            "If the name of the author is pogo and the current "
            "post author is pogo111, it will be downloaded "
            "this disables this feature, "
            "Default : False"
        )
    )
    # Clear database
    parser.add_argument(
        "--clear-database",
        action="store_true",
        default=False,
        help=(
            "If set, will remove the database, "
            "Default : False"
        )
    )
    # Don't use the database
    parser.add_argument(
        "--no-database",
        action="store_true",
        default=False,
        help=(
            "If set, this will download a new html file "
            "instead of appending to an existing one and not modify records "
            "in the database, "
            "Default : False"
        )
    )
    # Output dir
    parser.add_argument(
        '-o',
        '--output-dir',
        action="store",
        help="Output dir, Default is current dir",
        default=pathlib.Path.cwd(),
        type=pathlib.Path
    )
    args = parser.parse_args()
    return args


def contains_webarchive(archive_link: str) -> bool:
    return bool("web.archive.org" in archive_link)


def replace_youtube_embed(youtube_link: str) -> str:
    archive_link_parse = urlparse(youtube_link)
    video_id = archive_link_parse.path[
        archive_link_parse.path.rfind("/"):
    ][1:]
    video_component = f"/watch?v={video_id}"
    youtube_link = archive_link_parse._replace(
        path=video_component, query=""
    ).geturl()
    return youtube_link


def strip_webarchive_link(archive_link: str) -> str:
    archive_link_parsed = urlparse(archive_link)
    splitted_link = archive_link_parsed.geturl().split("/")[5:]
    new_link = "/".join(splitted_link)
    return new_link


def replace_youtube_frames(soup: 'BeautifulSoup') -> 'BeautifulSoup':
    for page in soup:
        current_post = page[0]
        frames = current_post.select(".embed-youtube > iframe")
        spans = current_post.select(".embed-youtube")
        bs = BeautifulSoup()
        for frame, span in zip(frames, spans):
            archive_link = frame.attrs["src"]
            embed_link = strip_webarchive_link(archive_link)
            youtube_link = replace_youtube_embed(embed_link)
            p = bs.new_tag("p")
            a = bs.new_tag("a", href=youtube_link)
            a.string = f"{youtube_link}"
            p.append(a)
            span.replace_with(p)
    return soup


def parse_input_links(links):
    links_file = pathlib.Path(links[0])
    if len(links) == 1 and links_file.exists():
        page_links = read_links(links_file)
    else:
        page_links = links
    return page_links



#!/usr/bin/python3

from typing import List, Optional
import sqlite3
import re
import pathlib
import logging

DB_PATH = pathlib.Path(__file__).parent.parent.parent / "risiparse.db"


def _replace_page_number(page_link: str) -> str:
    """
    The link stored in the database has a variable page number
    so we replace the the page number in the link by % to match
    the link.
    """
    page_link_number = [
              m.span() for m in re.finditer(r"\d*\d", page_link)
    ]
    page_link = (
              page_link[:page_link_number[3][0]]
              + '%'
    )
    return page_link


def create_db() -> None:
    con = sqlite3.connect(DB_PATH)
    try:
        with con:
            con.execute(
                '''create table if not exists risitas
                (id integer primary key autoincrement,
                domain varchar,
                title varchar,
                page_link varchar,
                file_path varchar,
                total_pages integer,
                current_page integer,
                message_cursor integer)'''
            )
            con.execute(
                '''create table if not exists authors
                (id integer primary key autoincrement,
                name varchar,
                risitas_id integer,
                foreign key(risitas_id) references risitas(id))'''
            )
    except sqlite3.OperationalError as e:
        logging.exception(e)
    con.close()


def read_db(page_link: str) -> List[Optional[tuple]]:
    create_db()
    con = sqlite3.connect(DB_PATH)
    page_link_normalized = _replace_page_number(page_link)
    try:
        with con:
            cursor = con.execute(
                '''select * from risitas where page_link LIKE ? limit 1''',
                (page_link_normalized, )
            )
            risitas_row = cursor.fetchone()
            rows = [risitas_row]
            if risitas_row:
                cursor_author = con.execute(
                    '''select name from authors where risitas_id = ?''',
                    (risitas_row[0], )
                )
                authors_row = cursor_author.fetchall()
                rows.append(authors_row)
    except sqlite3.OperationalError as e:
        logging.exception(e)
    con.close()
    return rows


def update_db(
        domain: str,
        title: str,
        page_link: str,
        file_path: 'pathlib.Path',
        total_pages: int,
        current_page: int,
        message_cursor: int,
        authors: List,
) -> None:
    create_db()
    # Need to handle the updates of existing rows
    file_path_str = str(file_path)
    con = sqlite3.connect(DB_PATH)
    page_link_normalized = _replace_page_number(page_link)
    cursor_existing_row = con.execute(
        '''select id from risitas where page_link like ? limit 1''',
        (page_link_normalized, )
    ).fetchone()
    if cursor_existing_row:
        try:
            with con:
                cursor = con.execute(
                    '''UPDATE risitas
                    SET domain = ?,
                    title = ?,
                    page_link = ?,
                    file_path = ?,
                    total_pages = ?,
                    current_page = ?,
                    message_cursor = ?
                    WHERE id = ?''',
                    (
                        domain,
                        title,
                        page_link,
                        file_path_str,
                        total_pages,
                        current_page,
                        message_cursor,
                        cursor_existing_row[0],
                    )
                )
                logging.info(
                    "A risitas has been updated in the database!"
                )
                logging.debug(
                    f"Id: {id} "
                    f"Title: {title} "
                    f"Current page: {current_page} "
                    f"Total pages : {total_pages}"
                )
                cursor_existing_row = con.execute(
                    '''select name from authors where id = ?''',
                    (cursor_existing_row[0], )
                ).fetchall()
                stored_authors = [author[0] for author in cursor_existing_row]
                for author in authors:
                    if author not in stored_authors:
                        con.execute(
                            '''INSERT INTO authors
                            (name, risitas_id)
                            VALUES (?, ?)''',
                            (
                                author,
                                cursor_existing_row[0],
                            )
                        )
                        logging.debug(
                            f"Author : {author} "
                            "has been inserted in the database!"
                        )
        except sqlite3.OperationalError as e:
            logging.exception(e)
    else:
        try:
            with con:
                cursor = con.execute(
                    '''INSERT INTO risitas
                    (domain,
                    title,
                    page_link,
                    file_path,
                    total_pages,
                    current_page,
                    message_cursor)
                    VALUES (?, ?, ?, ?, ?, ?, ?)''',
                    (
                        domain,
                        title,
                        page_link,
                        file_path_str,
                        total_pages,
                        current_page,
                        message_cursor,
                    )
                )
                logging.info(
                    "A new risitas has been inserted in the database!"
                )
                logging.debug(
                    f"Id: {cursor.lastrowid} "
                    f"Title: {title} "
                    f"Current page: {current_page} "
                    f"Total pages : {total_pages}"
                )
                for author in authors:
                    con.execute(
                        '''INSERT INTO authors
                        (name, risitas_id)
                        VALUES (?, ?)''',
                        (
                            author,
                            cursor.lastrowid,
                        )
                    )
                    logging.debug(
                        f"Author : {author} "
                        "has been inserted in the database!"
                    )
        except sqlite3.OperationalError as e:
            logging.exception(e)
    con.close()


def delete_db() -> None:
    DB_PATH.unlink()
    logging.info(f"Deleted database at {DB_PATH}")



#!/usr/bin/python3

"""This module takes care of the html to pdf conversion using QWebEngine"""

import os
import logging

from PyQt5.QtCore import QUrl
from PyQt5.QtGui import QPageLayout, QPageSize
from PyQt5 import QtWidgets, QtWebEngineWidgets


class PdfPage(QtWebEngineWidgets.QWebEnginePage):
    def __init__(self, output_dir):
        super().__init__()
        self._htmls = []
        self.output_dir = output_dir

        self.pdf_folder_path = output_dir / "risitas-pdf"

        self.settings = (
            QtWebEngineWidgets.QWebEngineSettings.globalSettings()
        )
        self.settings.setAttribute(
            QtWebEngineWidgets.QWebEngineSettings.JavascriptEnabled, False
        )

        self.setZoomFactor(1)
        self.layout = QPageLayout()
        self.layout.setPageSize(QPageSize(QPageSize.A4))
        self.layout.setOrientation(QPageLayout.Portrait)
        self.loadFinished.connect(self._handleLoadFinished)
        self.pdfPrintingFinished.connect(self._handlePrintingFinished)

    def convert(self, htmls):
        self._htmls = iter(htmls)
        self._fetchNext()

    def _fetchNext(self):
        try:
            self.current_file = next(self._htmls)
            self.load(QUrl.fromLocalFile(self.current_file.as_posix()))
        except StopIteration:
            return False
        return True

    def _handleLoadFinished(self):
        pdf_file = self.current_file.name[:-5] + ".pdf"
        output_file = str(self.pdf_folder_path) + os.sep + pdf_file
        self.printToPdf(output_file, pageLayout=self.layout)
        logging.info(f"Creating {output_file}")

    def _handlePrintingFinished(self):
        if not self._fetchNext():
            QtWidgets.QApplication.quit()
